use futures::Stream;
use uuid::Uuid;

use crate::Pool;

use super::permissions::PermissionType;

pub async fn insert(
    file_id: Uuid,
    parent_folder: Uuid,
    name: &str,
    size: i64,
    hash: Vec<u8>,
    pool: &Pool,
) -> sqlx::Result<()> {
    sqlx::query!("INSERT INTO files(file_id, folder_id, file_name, file_size, sha512) VALUES ($1, $2, $3, $4, $5)", file_id, parent_folder, name, size, hash)
        .execute(pool)
        .await
        .map(|_| ())
}

pub async fn update(file_id: Uuid, size: i64, hash: Vec<u8>, pool: &Pool) -> sqlx::Result<()> {
    sqlx::query!(
        "UPDATE files SET (sha512, file_size, updated_at) = ($2, $3, NOW()) WHERE file_id = $1",
        file_id,
        hash,
        size
    )
    .execute(pool)
    .await
    .map(|_| ())
}

#[derive(Debug, serde::Serialize)]
#[allow(clippy::struct_field_names, clippy::module_name_repetitions)]
pub struct FileWithoutParentId {
    pub file_id: Uuid,
    pub file_name: String,
    pub file_size: i64,
    pub sha512: String,
    pub created_at: chrono::NaiveDateTime,
    pub updated_at: chrono::NaiveDateTime,
}

pub fn get_files(
    folder_id: Uuid,
    pool: &Pool,
) -> impl Stream<Item = sqlx::Result<FileWithoutParentId>> + '_ {
    sqlx::query_as!(FileWithoutParentId, r#"SELECT file_id, file_name, file_size, encode(sha512, 'base64') as "sha512!", created_at, updated_at FROM files WHERE folder_id = $1"#, folder_id)
        .fetch(pool)
}

async fn get_folder_id(file_id: Uuid, pool: &Pool) -> sqlx::Result<Option<Uuid>> {
    let uuid = sqlx::query!("SELECT folder_id FROM files WHERE file_id = $1", file_id)
        .fetch_optional(pool)
        .await?
        .map(|record| record.folder_id);
    Ok(uuid)
}

pub async fn get_permissions(
    file_id: Uuid,
    user_id: i32,
    pool: &Pool,
) -> sqlx::Result<PermissionType> {
    match get_folder_id(file_id, pool).await? {
        Some(folder_id) => super::folder::get_permissions(folder_id, user_id, pool).await,
        None => Ok(PermissionType::default()),
    }
}

pub async fn get_name(file_id: Uuid, pool: &Pool) -> sqlx::Result<Option<String>> {
    let record = sqlx::query!("SELECT file_name FROM files WHERE file_id = $1", file_id)
        .fetch_optional(pool)
        .await?;
    Ok(record.map(|record| record.file_name))
}

pub async fn delete(file_id: Uuid, pool: &Pool) -> sqlx::Result<bool> {
    sqlx::query!("DELETE FROM files WHERE file_id = $1", file_id)
        .execute(pool)
        .await
        .map(|result| result.rows_affected() > 0)
}
