use std::collections::{HashMap, HashSet};

use axum::extract::multipart::{self, Multipart};
use futures::TryStreamExt;
use tokio::io::AsyncWrite;

use crate::prelude::*;

#[derive(Debug, Deserialize)]
pub struct Params {
    parent_folder: Uuid,
}

async fn create_file(
    file_id: Uuid,
    file: impl AsyncWrite + Unpin,
    file_name: &str,
    field: &mut multipart::Field<'_>,
    parent_folder: Uuid,
    pool: &Pool,
) -> bool {
    let (hash, size) = match crate::FileStorage::write_to_file(file, field).await {
        Ok(values) => values,
        Err(err) => {
            tracing::warn!(%err);
            return false;
        }
    };
    db::file::insert(file_id, parent_folder, file_name, size, hash, pool)
        .await
        .inspect_err(|err| tracing::warn!(%err))
        .is_ok()
}

pub async fn upload(
    Query(params): Query<Params>,
    State(state): State<AppState>,
    claims: Claims,
    mut multi: Multipart,
) -> Result<Json<HashMap<String, Uuid>>, StatusCode> {
    db::folder::get_permissions(params.parent_folder, claims.user_id, &state.pool)
        .await
        .handle_internal()?
        .can_write_guard()?;

    let existing_names: HashSet<String> = db::folder::get_names(params.parent_folder, &state.pool)
        .try_collect()
        .await
        .handle_internal()?;
    let mut result = HashMap::new();
    while let Ok(Some(mut field)) = multi.next_field().await {
        let Some(file_name) = field.file_name().map(ToOwned::to_owned) else {
            continue;
        };
        if existing_names.contains(&file_name) {
            continue;
        }
        let Ok((file_id, mut file)) = state.storage.create().await else {
            tracing::warn!("Couldn't create uuid for new file");
            continue;
        };
        let is_success = create_file(
            file_id,
            &mut file,
            &file_name,
            &mut field,
            params.parent_folder,
            &state.pool,
        )
        .await;
        if !is_success {
            let _ = state.storage.delete(file_id).await;
            continue;
        }
        result.insert(file_name, file_id);
    }

    Ok(Json(result))
}
